#this script pulls from kafka and calls rectvisconverter
from json import loads
import os
import requests
from .rectvisConverterHelper import GenerateAnnotation

class RectvisionConverter():
    def __init__(self, project_id, user_token, export_format):
        self.project_id = project_id
        self.user_token = user_token
        self.export_format = export_format

    def rectvision_converter(self):
        #get db info
        base_url = 'http://164.92.64.23/api/v1/project/'
        request_url = base_url + self.project_id
        headers={'Authorization': self.user_token}
        response = requests.get(request_url, headers=headers)
        current_project = loads(response.text)['data']
        annotations = current_project['annotations']
        labels = current_project['labels']
        if current_project['annotation_choice'] == 'Object_detection':
            shape_type = 'rectangle'
        train_ratio = current_project['training']['train_split']
        test_ratio = current_project['training']['test_split']
        valid_ratio = current_project['training']['validation_split']
        
        #convert data
        GenerateAnnotation(export_format=self.export_format, annotations=annotations, labels=labels, shape_type=shape_type,
                        database = 'User Provided', train_ratio=train_ratio, test_ratio=test_ratio, valid_ratio=valid_ratio)

        print('Annotations saved in train, test and validation folders!')




import getpass
import requests
from json import loads

class GetCredentials():
    def __init__(self, username = input('Enter Your Email: '), password = getpass.getpass('Enter Your Password: ')):
        self.username = username
        self.password = password

    def get_user_id(self):
        request_url = 'http://164.92.64.23/api/v1/auth/login'
        data={'email':self.username, 'password':self.password}
        response = requests.post(request_url, data=data)
        if response.ok:
            print("Successful login!")            
            response_details = loads(response.text)['data']
            user_id = response_details["user"]["_id"]
            token = response_details["token"]
            print('Use this user_id and token pair to validate this session: {}, {}'.format(user_id, token))
        else:
            print("Something went wrong! Make sure email and password entered are correct!")
            print(response.text)

    def get_project_ids(self, user_id, token):
        base_url = 'http://164.92.64.23/api/v1/project/all/'
        request_url = base_url + user_id
        headers={'Authorization':token}
        response = requests.get(request_url, headers=headers)
        if response.ok:
            print("Projects and their respective ids: \n")            
            response_details = loads(response.text)['data']
            #extract project details from response_details
            for project in response_details:
                project_name = project['name']
                project_id = project['_id']
                print('Project Name: {}    Project ID: {}'.format(project_name, project_id))            
        else:
            print("Something went wrong! Make sure user_id and token entered are correct!")
            print(response.text)

    def get_project_credential(self, project_id, token):
        base_url = 'http://164.92.64.23/api/v1/project/'
        request_url = base_url + project_id
        headers={'Authorization':token}
        response = requests.get(request_url, headers=headers)
        if response.ok:
            print("Project {} credentials: \n".format(project_id))            
            response_details = loads(response.text)['data']
            #extract project details from response_details
            print(response_details)            
        else:
            print("Something went wrong! Make sure project_id and token entered are correct!")
            print(response.text)


### code

import torch, torchvision
print(torch.__version__, torch.cuda.is_available())

# set GPU usage
device = torch.device('cuda:0')

# Some basic setup:
# Setup detectron2 logger
import detectron2, cv2
from tqdm import tqdm
from detectron2.utils.logger import setup_logger
setup_logger()

# import some common libraries
import numpy as np
import os, json, cv2, random, sys

# import some common detectron2 utilities
from detectron2 import model_zoo
from detectron2.engine import DefaultPredictor
from detectron2.config import get_cfg
from detectron2.utils.visualizer import Visualizer
from detectron2.data import MetadataCatalog, DatasetCatalog


cfg = get_cfg()
cfg.merge_from_file(model_zoo.get_config_file("COCO-InstanceSegmentation/mask_rcnn_R_50_FPN_3x.yaml"))
cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.7  # set threshold for this model
# Find a model from detectron2's model zoo. You can use the https://dl.fbaipublicfiles... url as well
cfg.MODEL.WEIGHTS = model_zoo.get_checkpoint_url("COCO-InstanceSegmentation/mask_rcnn_R_50_FPN_3x.yaml")



predictor = DefaultPredictor(cfg)
# outputs = predictor(im)


print('video dir: ', sys.argv[1])
cap = cv2.VideoCapture(sys.argv[1])

TotalFrames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
#define the codec and create videoWriter object
fourcc= cv2.VideoWriter_fourcc(*'XVID')
print((width, height))
out = cv2.VideoWriter('processed_video.avi', fourcc, 20.0, (1536, 864))


# def detectBox(outputs, frame):
#     color = (255, 0, 0)
#     instance = outputs['instances']
#     for count, (bbox, scores, class_name) in enumerate(zip(instance.pred_boxes, instance.scores, instance.pred_classes)):
#         bbox = bbox.to('cpu').numpy()
#         scores = scores.to('cpu').numpy()
#         class_name = class_name.to('cpu').numpy()
#         cv2.rectangle(frame, (int(bbox[0]), int(
#                 bbox[1])), (int(bbox[2]), int(bbox[3])), color, 2)
#         cv2.putText(frame, str(class_name)+ '_'+ str(count), (int(
#                 bbox[0]), int(bbox[1]-10)), 0, 0.5, (255, 255, 255), 1)

#     return frame


def detectBox(outputs, frame):
    v = Visualizer(frame[:,:,::-1], MetadataCatalog.get(cfg.DATASETS.TRAIN[0]), scale=1.2)
    out = v.draw_instance_predictions(outputs["instances"].to("cpu"))
    return out.get_image() #[:, :, ::-1]


for _, i in enumerate(tqdm(range(TotalFrames))):
    ret, frame = cap.read()
    if ret== True:
        # frame = cv2.flip(frame, 1)
        detections = predictor(frame)

        appendFrame = detectBox(detections, frame)
        # print(appendFrame.shape)
        appendFrame = cv2.cvtColor(appendFrame, cv2.COLOR_BGR2RGB)
        out.write(appendFrame)

        # cv2.imshow('frame', frame)

        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    else:
        break

cap.release()
out.release()
cv2.destroyAllWindows()


import numpy as np 
import cv2
import moviepy.editor
from moviepy.video.io.ffmpeg_tools import ffmpeg_extract_subclip
from moviepy.video.io.VideoFileClip import VideoFileClip, AudioFileClip
from .imageProcessor import ImageProcessor

class VideoProcessor():
    def __init__(self, video_path=None):
        if video_path is not None:
            self.video_path = video_path
            self.video = cv2.VideoCapture(video_path)
            self.imageProcessor = ImageProcessor()

    def videoFLipping(self, out_video_path, flip_dir='horizontal', video=None, video_path=None):
        '''
        This method flips every frame in the video in the direction specified by flip_dir
        and writes the resulting video to out_video_path.

        The parameters passed are:
        video: Video to flip.
        video_path: Path to video to flip.
        flip_dir: The direction to flip in ('horizontal' or 'vertical').
        out_video_path: The path to write the flipped video to. Can be .mp4 or .mov.

        '''
        if video is None:
            video = self.video
        if video_path is None:
            video_path = self.video_path

        new_frames = []
        flipped_frame_size = (0,0)
        fps = int(video.get(cv2.CAP_PROP_FPS))

        #extract audio from video
        self.mp4Tomp3('audio.mp3', video=VideoFileClip(video_path))
        

        while (video.isOpened()):
            ret, frame = video.read()
            if not ret:
                break
            flipped_frame = self.imageProcessor.imageFlipping(flip_dir=flip_dir, img=frame)
            new_frames.append(flipped_frame)
            flipped_frame_size = flipped_frame.shape[:2][::-1] #result as w, h

            #exit if escape key is pressed
            k = cv2.waitKey(30) & 0xff
            if k==27:
                break
        
        #release videocapture object
        video.release

        #create videowriter object
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        out_video = cv2.VideoWriter('video.mp4', fourcc , fps, flipped_frame_size)
        #write newly flipped frames to video
        for i in range(len(new_frames)):
            out_video.write(new_frames[i])
        #release videowriter object    
        out_video.release()

        video = moviepy.editor.VideoFileClip('video.mp4')
        video.write_videofile(out_video_path, codec='mpeg4', audio='audio.mp3')
                
        print('SUCCESSFULLY FLIPPED VIDEO!')
        
        

    def mp4Tomp3(self, out_audio_path, video=None):
        '''
        This method converts an mp4 video to audio and writes the resulting audio to out_audio_path.

        The parameters passed are:
        video: Video to convert to mp3.
        out_audio_path: The path to write the mp3 audio to.

        '''
        if video is None:
            video = moviepy.editor.VideoFileClip(self.video_path)
        video.audio.write_audiofile(out_audio_path)
        print('SUCCESSFULLY CONVERTED VIDEO TO MP3!')

    def mp4ToMov(self, mov_video_path, video=None):
        '''
        This method converts an mp4 video to an mov video and writes the 
        resulting video to out_video_path.

        The parameters passed are:
        video: Video to convert to mov.
        out_video_path: The path to write the mov video to.

        '''
        if video is None:
            video = moviepy.editor.VideoFileClip(self.video_path)
        video.write_videofile(mov_video_path, codec='mpeg4')
        print('SUCCESSFULLY CONVERTED VIDEO TO MOV!')

    def mp4ToAvi(self, avi_video_path, video=None):
        '''
        This method converts an mp4 video to an avi video and writes the 
        resulting video to out_video_path.

        The parameters passed are:
        video: Video to convert to mov.
        out_video_path: The path to write the avi video to.

        '''
        if video is None:
            video = moviepy.editor.VideoFileClip(self.video_path)
        video.write_videofile(avi_video_path, codec='png')
        print('SUCCESSFULLY CONVERTED VIDEO TO AVI!')

    def insertSignature(self, signature_path, out_video_path, signature_start=(0,0), signature_size=None, video=None, video_path=None):
        '''
        This method inserts a signature to all frames of a video.

        The parameters passed are:
        signature_path: Path to the signature image to be inserted.
        out_video_path: Path to write signed video to. Can be .mp4 or .mov.
        signature_start: Tuple containing the starting coordinates of the label 
                         on the video frame in the form(h,w).
        signature_size: Tuple containing the size of the signature in the form (h,w)
        video: Video to insert signature.
        video_path: Path to video to insert signature.

        '''
        if video is None:
            video = self.video
        if video_path is None:
            video_path = self.video_path
        new_frames = []
        added_frame_size = (0,0)
        fps = int(video.get(cv2.CAP_PROP_FPS))

        #extract audio from video
        self.mp4Tomp3('audio.mp3', video=VideoFileClip(video_path))

        while (video.isOpened()):
            ret, frame = video.read()
            if not ret:
                break
            added_frame = self.imageProcessor.insertLabel(signature_path, label_start=signature_start, label_size=signature_size, img=frame)
            new_frames.append(added_frame)
            added_frame_size = added_frame.shape[:2][::-1] #result as w, h

            #exit if escape key is pressed
            k = cv2.waitKey(30) & 0xff
            if k==27:
                break
        
        #release videocapture object
        video.release

        #create videowriter object
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        out_video = cv2.VideoWriter('video.mp4', fourcc , fps, added_frame_size)
        #write newly labelled frames to video
        for i in range(len(new_frames)):
            out_video.write(new_frames[i])
        #release videowriter object    
        out_video.release()

        #add audio to newly created video
        video = moviepy.editor.VideoFileClip('video.mp4')
        video.write_videofile(out_video_path, codec='mpeg4', audio='audio.mp3')

        print('SUCCESSFULLY INSERTED SIGNATURE TO VIDEO!')
        
    def insertLabel(self, out_video_path, label, label_color=(0,0,255), label_position=(20,20), font_scale=1.0, video=None, video_path=None):
        '''
        This method inserts a text label to all frames of a video.

        The parameters passed are:
        out_video_path: Path to write labelled video to. Can be .mp4 or .mov.
        label: Text to be inserted.
        label_color: Colour of text as tuple in the form (B, G, R) and values between 0 and 255.
        label_positioin: Tuple containing the coordinates of the bottom-left corner of label 
                         on the video frame in the form (x, y).
        font_scale: Scale to be multiplied by base-size of font type
        video: Video to insert text label.
        video_path: Path to video to insert text label.
        
        '''
        if video is None:
            video = self.video
        if video_path is None:
            video_path = self.video_path
        new_frames = []
        label_font = cv2.FONT_ITALIC
        labelled_frame_size = (0,0)
        fps = int(video.get(cv2.CAP_PROP_FPS))

        #extract audio from video
        self.mp4Tomp3('audio.mp3', video=VideoFileClip(video_path))

        while (video.isOpened()):
            ret, frame = video.read()
            if not ret:
                break
            labelled_frame = cv2.putText(img=frame, text=label, org=label_position, fontFace=label_font, fontScale=font_scale,color=label_color, thickness=5 )
            new_frames.append(labelled_frame)
            labelled_frame_size = labelled_frame.shape[:2][::-1] #result as w, h

            #exit if escape key is pressed
            k = cv2.waitKey(30) & 0xff
            if k==27:
                break
        
        #release videocapture object
        video.release

        #create videowriter object
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        out_video = cv2.VideoWriter('video.mp4', fourcc , fps, labelled_frame_size)
        #write newly labelled frames to video
        for i in range(len(new_frames)):
            out_video.write(new_frames[i])
        #release videowriter object    
        out_video.release()

        #add audio to newly created video
        video = moviepy.editor.VideoFileClip('video.mp4')
        video.write_videofile(out_video_path, codec='mpeg4', audio='audio.mp3')

        print('SUCCESSFULLY INSERTED LABELS TO VIDEO!')
    
    def noiseCancellation(self):
        pass

    def trimVideo(self, start_trim, end_trim, trimmed_video_path, video_path=None):
        '''
        This method trims a video to span between start_trim and end_trim.

        The parameters passed are:
        start_trim: TIme mark to start the trim (in seconds).
        end_trim: Time mark to end the trim (in seconds).
        trimmed_video_path: Path to write trimmed video to.
        video_path: Path to video to be trimmed.
        
        '''
        if video_path is None:
            video_path = self.video_path
        ffmpeg_extract_subclip(filename=video_path, t1=start_trim, t2=end_trim, targetname=trimmed_video_path)
        print('SUCCESSFULLY TRIMED VIDEO!')

    def mergeVideo(self, add_vid_path, out_video_path, main_vid=None):
        '''
        This method merges two videos together.

        The parameters passed are:
        add_vid_path: Path to video to be added to a main video.
        out_video_path: Path to write merged video to.
        main_vid: Video to add another video to.

        '''
        if main_vid is None:
            main_vid = moviepy.editor.VideoFileClip(self.video_path)
        add_vid = moviepy.editor.VideoFileClip(add_vid_path)

        merged_vid = moviepy.editor.concatenate_videoclips([main_vid, add_vid])
        merged_vid.write_videofile(out_video_path, codec='libx264')
        print('SUCCESSFULLY MERGED VIDEOS!')


###this script returns all credentials linked to a particular project
import requests
from json import loads

class GetProjectCredential():
    def __init__(self, project_id, token):
        self.project_id = project_id
        self.token = token

    def get_project_credential(self):
        base_url = 'http://164.92.64.23/api/v1/project/'
        request_url = base_url + self.project_id
        headers={'Authorization':self.token}
        response = requests.get(request_url, headers=headers)
        if response.ok:
            print("Project {} credentials: \n".format(self.project_id))            
            response_details = loads(response.text)['data']
            #extract project details from response_details
            
        else:
            print("Something went wrong! Make sure project_id and token entered are correct!")
            print(response.text)

def get_project_credential(project_id, token):
    project = GetProjectCredential(project_id, token)
    project.get_project_credential()


# from .s3API import s3API
import requests
from json import loads, dump

# class Download():
#     def __init__(self, user_id, project_id, filename, output_filepath):
#         self.user_id = user_id
#         self.project_id = project_id
#         self.filename = filename #name of file stored in aws bucket
#         self.output_filepath = output_filepath #name to be used to save downloaded file

#         #download the data
#         self.download()

#     def download(self):
#         """This method downloads data from the aws storage
#         of this specified project_id"""
         
#         s3API.downloader(self.filename, self.output_filepath)
#         print('{} downloaded successfully and saved to {}'.format(self.filename, self.output_filepath))

class Download():
    def __init__(self, project_id, user_token):
        self.project_id = project_id
        self.user_token = user_token
        
    def download(self):
        #get project details from endpoint
        base_url = 'http://164.92.64.23/api/v1/project/'
        request_url = base_url + self.project_id
        headers={'Authorization': self.user_token}
        response = requests.get(request_url, headers=headers)
        if response.ok:            
            current_project = loads(response.text)['data']
            annotations = current_project['annotations']
            with open('annotations.json', 'w') as f:
                dump(annotations, f)
            labels = current_project['labels']    
            with open('labels.txt', 'w') as f:
                dump(labels, f)
            print('Annotations saved to annotations.json and labels saved to labels.txt')
            
        else:
            print("Something went wrong! Make sure project_id and token entered are correct!")
            print(response.text)




import json
import os
import numpy as np
import cv2
import random
import detectron2
from detectron2 import model_zoo
from detectron2.config import get_cfg
from detectron2.util.logger import setup_logger
from detectron2.data.datasets import register_coco_instances
from detectron2.engine import DefaultTrainer, DefaultPredictor
from detectron2.evaluation import COCOEvaluator, inference_on_dataset
from detectron2.data import build_detection_test_loader

class FasterRCNN():
    def __init__(self, num_classes, batch_size, num_epochs):
        self.num_classes = num_classes
        self.batch_size = batch_size
        self.num_epochs = num_epochs
        self.working_dir = os.path.join(os.path.dirname(__file__), 'project')      
        self.valid_annotations = os.path.join(self.working_dir,'valid/annotations.json')
        self.train_annotations = os.path.join(self.working_dir,'train/annotations.json')
        self.test_annotations = os.path.join(self.working_dir, 'test/annotations.json')
        self.valid_images = os.path.join(self.working_dir,'valid/images')
        self.train_images = os.path.join(self.working_dir,'train/images')
        self.test_images = os.path.join(self.working_dir, 'test/images')

    def register_data(self):
        for data_category, dataset in {'train_data':[self.train_annotations, self.train_images], 
                                  'test_data':[self.test_annotations, self.test_images], 
                                  'valid_data':[self.valid_annotations, self.valid_images]}.items():
            register_coco_instances(data_category, {}, dataset[0], dataset[1])
    
    def edit_config(self):
        cfg = get_cfg()
        cfg.merge_from_file(model_zoo.get_config_file("COCO-Detection/faster_rcnn_X_101_32x8d_FPN_3x.yaml"))
        cfg.DATASETS.TRAIN = ("train_data")
        cfg.DATASETS.TEST = ("valid_data")
        cfg.MODEL.WEIGHTS = model_zoo.get_checkpoint_url("COCO-Detection/faster_rcnn_X_101_32x8d_FPN_3x.yaml")  # Let training initialize from model zoo
        cfg.SOLVER.IMS_PER_BATCH = 2
        cfg.SOLVER.BASE_LR = 0.00025  
        cfg.SOLVER.MAX_ITER = self.num_epochs    
        cfg.SOLVER.STEPS = []        # do not decay learning rate
        cfg.MODEL.ROI_HEADS.BATCH_SIZE_PER_IMAGE = 128  
        cfg.MODEL.ROI_HEADS.NUM_CLASSES = self.num_classes
        cfg.TEST.DETECTIONS_PER_IMAGE = 20
        cfg.OUTPUT_DIR = os.path.join(self.working_dir, 'output')

        os.makedirs(cfg.OUTPUT_DIR, exist_ok=True)
        return cfg
        
    def train(self):
        self.register_data()
        self.model_cfg = self.edit_config()
        trainer = DefaultTrainer(self.model_cfg) 
        trainer.resume_or_load(resume=False)
        trainer.train() 

    def evaluate(self):
        self.model_cfg.MODEL.WEIGHTS = os.path.join(self.model_cfg.OUTPUT_DIR, "model_final.pth") 
        self.model_cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.7   # set a custom testing threshold
        predictor = DefaultPredictor(self.model_cfg)
        evaluator = COCOEvaluator("test_data", output_dir="./output")
        val_loader = build_detection_test_loader(self.model_cfg, "test_data")
        return inference_on_dataset(predictor.model, val_loader, evaluator)






###this script returns all projects (names and respective project_id) for the user to choose from
import requests
from json import loads

class GetProjectIds():
    def __init__(self, user_id, token):
        self.user_id = user_id
        self.token = token

    def get_project_ids(self):
        base_url = 'http://164.92.64.23/api/v1/project/all/'
        request_url = base_url + self.user_id
        headers={'Authorization':self.token}
        response = requests.get(request_url, headers=headers)
        if response.ok:
            print("Projects and their respective ids: \n")            
            response_details = loads(response.text)['data']
            #extract project details from response_details
            
        else:
            print("Something went wrong! Make sure user_id and token entered are correct!")
            print(response.text)

# def get_user_id(user_id, token):
#     user = GetProjectIds(user_id, token)
#     user.get_project_ids()


import numpy as np 
import cv2
import PIL
from PIL import Image
from sklearn.metrics import pairwise
from statistics import mean

class ImageProcessor():
    def __init__(self, image_path=None):
        if image_path is not None:
            self.image_pil = Image.open(image_path)
            self.image_array = cv2.imread(image_path)
            self.image_height, self.image_width = self.image_array.shape[0:2]

    def imageFlipping(self, flip_dir='horizontal', img=None):
        '''
        This method flips image in the direction specified by flip_dir.

        The parameter passed is:
        img: Image to flip.
        flip_dir: The direction to flip in ('horizontal' or 'vertical').

        The parameter returned is:
        img_flip: Flipped image as numpy ndarray 
        '''
        if img is None:
            img = self.image_array
        if flip_dir=='horizontal':
            img_flip = cv2.flip(img, 1)
        elif flip_dir=='vertical':
            img_flip = cv2.flip(img, 0)
            
        return img_flip

    def imageRotationWithoutAngle(self, rotation_dir='clockwise', img=None):
        '''
        This method rotates image in the direction specified by rotation_dir.

        The parameter passed is:
        rotation_dir: The direction to rotate ('clockwise' or 'anticlockwise').
        img: Image to rotate.

        The parameter returned is:
        img_rot: Rotated image as numpy ndarray 
        '''
        if img is None:
            img = self.image_array
        if rotation_dir=='clockwise':
            img_rot = cv2.rotate(img, cv2.ROTATE_90_CLOCKWISE)
        elif rotation_dir=='anticlockwise':
            img_rot = cv2.rotate(img, cv2.ROTATE_90_COUNTERCLOCKWISE)
            
        return img_rot

    def imageRotationWithAngle(self, rotation_angle=0, img=None):
        '''
        This method rotates image in the direction specified by rotation_angle.

        The parameter passed is:
        rotation_angle: The direction to rotate (0≤rotation_angle≤360).
        img: Image to rotate

        The parameter returned is:
        img_rot: Rotated image as pil image
        '''
        if img is None:
            img = self.image_pil
        img_rot = img.rotate(rotation_angle, expand=True)
        return img_rot

    def imageResizing(self, new_dim=None, img=None):
        '''
        This method resizes image to dimension specified by new_dim.

        The parameter passed is:
        new_dim: A tuple containing resized image dimension in the form (h,w).
        img: Image to resize.

        The parameter returned is:
        img_resized: Resized image as numpy ndarray
        '''
        if img is None:
            img = self.image_array
        if new_dim is None:
            img_height, img_width = img.shape[0:2]
            new_dim = (img_height, img_width)
               
        img_resized = cv2.resize(img, new_dim[::-1])
        return img_resized

    def imageScaling(self, scale=1, img=None):
        '''
        This method scales image based on the specified scale value.

        The parameter passed is:
        scale: The scale factor for which the image should be scaled. 
               If 0≤scale≤1, image is reduced, if scale>1, image is enlarged.

        The parameter returned is:
        img_scaled: Scaled image as numpy ndarray
        '''
        if img is None:
            img = self.image_array
        img_height, img_width = img.shape[0:2]
        if scale:            
            height = int(img_height*scale)
            width = int(img_width*scale)
            new_dim = (height, width)
        
        img_scaled = cv2.resize(img, new_dim[::-1])
        return img_scaled

    def imageCropping(self, crop_start = (0,0), crop_end=None, img=None):
        '''
        This method crops out a portion of an image based on the specified 
        crop_start and crop_end values.

        The parameters passed are:
        crop_start: Tuple containing the starting coordinates of the crop in the form (w,h).
        crop_end: Tuple containing the stop coordinates of the crop in the form (w,h)

        The parameter returned is:
        img_crop: Cropped image as numpy ndarray
        '''
        if img is None:
            img = self.image_array
        img_height, img_width = img.shape[0:2]
        if crop_end is None:
            crop_end = (img_width, img_height)
        img_crop = img[crop_start[1]:crop_end[1], crop_start[0]:crop_end[0]]
        return img_crop

    def faceDetection(self, img=None):
        '''
        This method detects and crops out human faces present in an image.
        
        The parameters passed is:
        img: Image from which to detect faces.

        The parameter returned is:
        faces: List of all detected images as numpy ndarrays
        '''
        if img is None:
            img = self.image_array
        
        face_cascade = cv2.CascadeClassifier(cv2.data.haarcascades + 'haarcascade_frontalface_default.xml')
        img_gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        face_coordinates = face_cascade.detectMultiScale(img_gray, 1.05, 4)
        faces = []
        faces_w = []
        faces_h = []
        for (x, y, w, h) in face_coordinates:
            faces.append(self.imageCropping((x, y), (x+w, y+h)))  
            faces_w.append(w)
            faces_h.append(h)  

        #get average width and height of images
        out_face_size = (mean(faces_h), mean(faces_w))
        #resize all faces to average sizes
        for idx, face in enumerate(faces):
            if face.shape[:2] != out_face_size:
                faces[idx] = cv2.resize(face, out_face_size[::-1])
        #create a vertical collage of all detected images
        face_collage = np.vstack(faces)    
        return face_collage, faces, face_coordinates

    def insertLabel(self, label_path, label_start=(0,0), label_size=None, img=None):
        '''
        This method inserts a label to an image.

        The parameters passed are:
        img: Image to add label to.
        label_path: path to the label to be inserted.
        label_start: Tuple containing the starting coordinates of the label 
                     on the original image in the form(h,w).
        label_size: Tuple containing the size of the label in the form (h,w)

        The parameter returned is:
        added_image: Image with label added as numpy ndarray
        '''
        if img is None:
            img = self.image_array 
                  
        label = cv2.imread(label_path)
        
        if label_size is None:
            label_size = label.shape[:2] #result is in h,w,c
        else:
            label = cv2.resize(label, label_size[::-1])
        
        added_image = img
        label_end = (label_start[0]+label_size[0], label_start[1]+label_size[1])
        roi = img[label_start[0]:label_end[0], label_start[1]:label_end[1]]
        label = cv2.addWeighted(roi, 0, label, 1, 0.0)
        added_image[label_start[0]:label_end[0], label_start[1]:label_end[1]] = label
        
        return added_image
    
    def findFace(self, ref_face_path, thresh=0.9, img=None):
        '''
        This method finds a face in an image.

        The parameters passed are:
        ref_face_path: Path to the reference face to be found.
        thresh: Minimum simiilarity value. Must be a value between 0 and 1.
        img: Image to look for faces in.

        The parameters returned are:
        found_face: Face found as numpy ndarray
        similarity_percentage: Percentage similarity between reference face and found face. 
        '''
        if img is None:
            img=self.image_array

        ref_face = cv2.imread(ref_face_path)
        _, ref_face, _ = self.faceDetection(ref_face)
        #convert to greyscale to remove colour bias in comparison
        ref_face = cv2.cvtColor(ref_face[0], cv2.COLOR_BGR2GRAY)
        ref_face_size = ref_face.shape
        #flatten ref_face
        ref_face = ref_face.reshape(1, -1)
        _, faces, face_coordinates = self.faceDetection()
        similarities = []
        #check faces for ref_image and return true based on threshold
        for face in faces:
            #convert face to grayscale
            face = cv2.cvtColor(face, cv2.COLOR_BGR2GRAY)
            #resize face to ref_face size.
            face = cv2.resize(face, ref_face_size[:2])
            #flatten face
            face = face.reshape(1, -1)
            #compute cosine similarity
            similarity = pairwise.cosine_similarity(face, ref_face)
            similarities.append(similarity)
        
        max_similarity_idx = np.argmax(similarities)
        if similarities[max_similarity_idx] >= thresh:
            similarity_percentage = similarities[max_similarity_idx] * 100
            found_face = faces[max_similarity_idx]
            #get face coordinates
            found_face_coordinates = face_coordinates[max_similarity_idx]
            #define individual coordinates for found face
            found_face_x, found_face_y = found_face_coordinates[:2]
            found_face_w, found_face_h = found_face_coordinates[2:]
            #draw red bounding box around found face in original image
            cv2.rectangle(img, (found_face_x, found_face_y), (found_face_x+found_face_w, found_face_y+found_face_h), (0,0,255))
            
            return img, similarity_percentage
        print('FACE NOT FOUND')
        
        
    


###this script takes in user id and password, and returns token
###to be used in the other modules (data download, model training)
import getpass
import requests
from json import loads

class GetUserId():
    def __init__(self, username = input('Enter Your Email: '), password = getpass.getpass('Enter Your Password: ')):
        self.username = username
        self.password = password

    def get_user_id(self):
        request_url = 'http://164.92.64.23/api/v1/auth/login'
        data={'email':self.username, 'password':self.password}
        response = requests.post(request_url, data=data)
        if response.ok:
            print("Successful login!")            
            response_details = loads(response.text)['data']
            user_id = response_details["user"]["_id"]
            token = response_details["token"]
            print('Use this user_id and token pair to validate this session: {}, {}'.format(user_id, token))
        else:
            print("Something went wrong! Make sure email and password entered are correct!")
            print(response.text)

# def get_user_id():
#     user = GetUserId()
#     user.login()


import subprocess
import os
import sys
import yaml

class Yolov5():
    def __init__(self, num_classes, img_size, batch_size, num_epochs, labels, project_name):
        self.num_classes = num_classes
        self.img_size = img_size
        self.batch_size = batch_size
        self.num_epochs = num_epochs
        self.labels = labels
        self.project_name = project_name
        self.working_dir = os.path.dirname(__file__)

    def setup(self):
        self.git_clone("https://github.com/ultralytics/yolov5")
        os.chdir(os.path.join(self.working_dir, "yolov5"))
        subprocess.check_call([sys.executable, "-m", "pip", "install", "-r", "requirements.txt"])

    def git_clone(self, url):
        subprocess.check_output(["git",  "clone", url])
    
    def create_data_config(self):
        configs = {"train": "../dataset/train/images",
                   "val": "../dataset/valid/images",
                   "test": "../dataset/test/images",
                   "nc": self.num_classes,
                   "names": self.labels
                  }
        self.data_yaml = os.path.join(self.working_dir, "yolov5/data", "data.yaml")
        with open(self.data_yaml, "w") as file:
            yaml.dump(configs, file, default_flow_style=None)    

    def train(self):
        self.setup()
        self.create_data_config()
        self.train_model = os.path.join(self.working_dir,"yolov5/train.py")
        
        subprocess.call(["Python", self.train_model, 
                        "--img", str(self.img_size),
                        "--cfg", "yolov5s.yaml",
                        "--hyp", "hyp.scratch-low.yaml",
                        "--batch", str(self.batch_size),
                        "--epochs", str(self.num_epochs),
                        "--data", self.data_yaml,
                        "--weights", "yolov5s.pt",
                        "--workers", "24",
                        "--name", self.project_name])
    def get_map(self):
        self.test_model = os.path.join(self.working_dir,"yolov5/test.py")
        self.model_weights = os.path.join(self.working_dir,"yolov5/runs/train", self.project_name, "weights/best.pt")
        subprocess.call(["Python", self.test_model,
                        "--weights", self.model_weights,
                        "--data", self.data_yaml,
                        "--task", "test",
                        "--workers", "24",
                        "--name", "yolo_det"])

    
# num_classes, img_size, batch_size, num_epochs, labels, project_name = 3, 512, 2, 2, ["displeased", "laughing", "neutral"], "test_project"
# model = Model(num_classes, img_size, batch_size, num_epochs, labels, project_name)
# model.setup()
# model.create_data_config()
# model.train()
    
        




import numpy as np
from zipfile import ZipFile
import json
import random
import glob
import requests
import os
import csv
import ast
import xml.etree.ElementTree as ET


class GenerateAnnotation():   
    def __init__(self, export_format, annotations, labels=[], shape_type='rectangle', 
                database = 'User Provided', train_ratio=0.7, test_ratio=0.2, valid_ratio=0.1):
        self.annotations = annotations
        self.export_format = export_format
        self.shape_type = shape_type
        self.xml_template = os.path.join('templates', 'xmlTemplate.xml')
        self.database = database
        self.train_ratio = train_ratio
        self.test_ratio = test_ratio
        self.valid_ratio = valid_ratio
        self.ppts = []
        self.labels = labels
        #generate labelmap
        self.label_to_id_file_path = 'labelmap.txt'
        self.generate_labelmap(self.labels, self.label_to_id_file_path)
        self.label_to_id = self.read_dictionary(self.label_to_id_file_path)

        #insert which annotation conversion method should be called
        if self.export_format == 'labelme-json':
            self.rectjson_to_json()
        elif self.export_format == 'darknet-txt':
            self.rectjson_to_darknetTxt()
        elif self.export_format == 'yolo-txt':
            self.rectjson_to_yoloTxt()
        elif self.export_format == 'xml':
            self.rectjson_to_xml()
        elif self.export_format == 'yolov3-keras-txt':
            self.rectjson_to_yolov3kerasTxt()
        elif self.export_format == 'keras-retinanet-csv':
            self.rectjson_to_kerasRetinanetCsv()
        elif self.export_format == 'tf-csv':
            self.rectjson_to_tfCsv()

        
    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def generate_labelmap(self, labels, label_to_id_file_path='label2id.txt'):
        label_ids = list(range(0, len(labels)))
        label_to_id = dict(zip(labels, label_ids))

        with open(label_to_id_file_path, 'w') as data:
            data.write(str(label_to_id))
    
    def read_dictionary(self, dict_file_path):
        with open(dict_file_path, "r") as data:
            dictionary = ast.literal_eval(data.read())  
        return dictionary 

    def extract_info_from_json(self):
        self.ppts = []
        for image_data in self.annotations:
            current_img_path = image_data['image_url']
            current_img_width, current_img_height, current_img_depth = image_data['image_depth'], image_data['image_height'], 3
            annotations = image_data['annotation']
            image_ppts = []

            if self.export_format == 'labelme-json':
                for label in annotations:
                    #get list of annotations for label
                    label_annotations = annotations[label]
                    for points in label_annotations:
                        x_min, y_min, x_max, y_max = points[0], points[1] - points[3], points[0] + points[3], points[1]
                        image_ppts.append([current_img_path, label, self.shape_type, [[x_min, y_min], [x_max, y_max]]])
                self.ppts.append(image_ppts)
                
            
            elif self.export_format == 'darknet-txt':
                for label in annotations:
                    #get list of annotations for label
                    label_id = self.label_to_id[label]
                    label_annotations = annotations[label]
                    for points in label_annotations:
                        x_min, y_min = points[0], points[1] - points[3]
                        width, height = points[2], points[3]
                        x_center, y_center,  = x_min + (width/2), y_min + (height/2)
                        image_ppts.append([current_img_path, label_id, 
                                            np.abs(x_center)/current_img_width, 
                                            np.abs(y_center)/current_img_height, 
                                            np.abs(width)/current_img_width, 
                                            np.abs(height)/current_img_height, '\n' ])
                self.ppts.append(image_ppts)

            elif self.export_format == 'yolo-txt':
                for label in annotations:
                    #get list of annotations for label
                    label_id = self.label_to_id[label]
                    label_annotations = annotations[label]
                    for points in label_annotations:
                        x_min, y_min = points[0], points[1] - points[3]
                        width, height = points[2], points[3]
                        image_ppts.append([current_img_path, label_id, x_min, y_min, width, height, '\n' ])
                self.ppts.append(image_ppts)

            elif self.export_format == 'yolov3-keras-txt':
                for label in annotations:
                    #get list of annotations for label
                    label_id = self.label_to_id[label]
                    label_annotations = annotations[label]
                    for points in label_annotations:
                        x_min, y_min, x_max, y_max = points[0], points[1] - points[3], points[0] + points[3], points[1]
                        image_ppts.append([current_img_path, x_min, y_min, x_max, y_max, label_id])
                self.ppts.append(image_ppts)
            
            elif self.export_format == 'xml':
                for label in annotations:
                    #get list of annotations for label
                    label_annotations = annotations[label]
                    for points in label_annotations:
                        x_min, y_min, x_max, y_max = points[0], points[1] - points[3], points[0] + points[3], points[1]
                        image_ppts.append([current_img_path, current_img_width, 
                                            current_img_height, current_img_depth,
                                            label, self.shape_type, x_min, x_max, y_min, y_max])
                self.ppts.append(image_ppts)

            elif self.export_format == 'keras-retinanet-csv':
                for label in annotations:
                    #get list of annotations for label
                    label_annotations = annotations[label]
                    for points in label_annotations:
                        x_min, y_min, x_max, y_max = points[0], points[1] - points[3], points[0] + points[3], points[1]
                        image_ppts.append([current_img_path, x_min, y_min, x_max, y_max, label])
                self.ppts.append(image_ppts)

            elif self.export_format == 'tf-csv':
                for label in annotations:
                    #get list of annotations for label
                    label_annotations = annotations[label]
                    for points in label_annotations:
                        x_min, y_min, x_max, y_max = points[0], points[1] - points[3], points[0] + points[3], points[1]
                        image_ppts.append([current_img_path, label,
                                            current_img_width, current_img_height,
                                            x_min, y_min, x_max, y_max])
                self.ppts.append(image_ppts)

                    


    def pointsTobbox(self, points):
        '''This method converts the points array to the properties of the object's bounding box'''
        x_min, y_min = points[0], points[1] - points[3]
        x_max, y_max = points[0] + points[3], points[1]
        return [[x_min, y_min], [x_max, y_max]]

    def valid_path(self, path):
        isExist = os.path.exists(path)
        if not isExist:
            os.makedirs(path)
        return path

    def split(self, arr, split1, split2, split3):
        random.shuffle(arr)
        arr_length = len(arr)
        arr1 = arr[:round(split1*arr_length)]
        arr2 = arr[round(split1*arr_length):round((split1+split2)*arr_length)]
        arr3 = arr[round((split1+split2)*arr_length):]
        return arr1, arr2, arr3

    def rectjson_to_json(self):
        print('Starting conversion...')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        for image_ppt in self.ppts_train:
            
            output_json_dict = {
                'shapes': [],
                'imagePath': ''
            }
            for ppt in image_ppt:
                img_path, label, shape_type, points = ppt
                #update imagePath
                output_json_dict['imagePath']=img_path
                #shapes info
                shapes_info = {
                    'label':'',
                    'points': [],
                    'shape_type': ''
                }
                #update shapes_info
                shapes_info['label'] = label
                shapes_info['points'] = points
                shapes_info['shape_type'] = shape_type
                output_json_dict['shapes'].append(shapes_info)
            #write to json file
            train_out_annotation_dir = self.valid_path('train')
            out_annotation_path = os.path.splitext(os.path.join(train_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.json'
            with open(out_annotation_path, 'w') as f:
                output_json = json.dumps(output_json_dict)
                f.write(output_json)
        for image_ppt in self.ppts_test:
            
            output_json_dict = {
                'shapes': [],
                'imagePath': ''
            }
            for ppt in image_ppt:
                img_path, label, shape_type, points = ppt
                #update imagePath
                output_json_dict['imagePath']=img_path
                #shapes info
                shapes_info = {
                    'label':'',
                    'points': [],
                    'shape_type': ''
                }
                #update shapes_info
                shapes_info['label'] = label
                shapes_info['points'] = points
                shapes_info['shape_type'] = shape_type
                output_json_dict['shapes'].append(shapes_info)
            #write to json file
            test_out_annotation_dir = self.valid_path('test')
            out_annotation_path = os.path.splitext(os.path.join(test_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.json'
            with open(out_annotation_path, 'w') as f:
                output_json = json.dumps(output_json_dict)
                f.write(output_json)
        for image_ppt in self.ppts_valid:
            
            output_json_dict = {
                'shapes': [],
                'imagePath': ''
            }
            for ppt in image_ppt:
                img_path, label, shape_type, points = ppt
                #update imagePath
                output_json_dict['imagePath']=img_path
                #shapes info
                shapes_info = {
                    'label':'',
                    'points': [],
                    'shape_type': ''
                }
                #update shapes_info
                shapes_info['label'] = label
                shapes_info['points'] = points
                shapes_info['shape_type'] = shape_type
                output_json_dict['shapes'].append(shapes_info)
            #write to json file
            validation_out_annotation_dir = self.valid_path( 'validation')
            out_annotation_path = os.path.splitext(os.path.join(validation_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.json'
            with open(out_annotation_path, 'w') as f:
                output_json = json.dumps(output_json_dict)
                f.write(output_json)
        print('All done!')

    def rectjson_to_darknetTxt(self):
        print('Starting conversion...')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        for image_ppt in self.ppts_train:            
            #write to txt file
            train_out_annotation_dir = self.valid_path('train')
            out_annotation_path = os.path.splitext(os.path.join(train_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.txt'
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        for image_ppt in self.ppts_test:            
            #write to txt file
            test_out_annotation_dir = self.valid_path('test')
            out_annotation_path = os.path.splitext(os.path.join(test_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.txt'
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        for image_ppt in self.ppts_valid:            
            #write to txt file
            validation_out_annotation_dir = self.valid_path('validation')
            out_annotation_path = os.path.splitext(os.path.join(validation_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.txt'
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        print('All done!') 

    def rectjson_to_yoloTxt(self):
        print('Starting conversion...')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        for image_ppt in self.ppts_train:            
            #write to txt file
            train_out_annotation_dir = self.valid_path('train')
            out_annotation_path = os.path.splitext(os.path.join(train_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.txt'
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        for image_ppt in self.ppts_test:            
            #write to txt file
            test_out_annotation_dir = self.valid_path('test')
            out_annotation_path = os.path.splitext(os.path.join(test_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.txt'
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        for image_ppt in self.ppts_valid:            
            #write to txt file
            validation_out_annotation_dir = self.valid_path('validation')
            out_annotation_path = os.path.splitext(os.path.join(validation_out_annotation_dir, image_ppt[0][0][8:]))[0] + '.txt'
            with open(out_annotation_path, 'w') as f:
                for ppts in image_ppt:
                    f.write(' '.join(str(ppt) for ppt in ppts[1:]))
        print('All done!')  

    def rectjson_to_yolov3kerasTxt(self):
        print('Starting conversion...')
        #write to txt file
        train_out_annotation_dir = self.valid_path('train')
        out_annotation_path = os.path.join(train_out_annotation_dir, 'annotations.txt')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        with open(out_annotation_path, 'w') as f:
            for image_ppt in self.ppts_train:            
                f.writelines([image_ppt[0][0], ' ', ' '.join(str(','.join(str(ppt) for ppt in ppts[1:])) 
                                                                for ppts in image_ppt), '\n'])
        #write to txt file
        test_out_annotation_dir = self.valid_path('test')
        out_annotation_path = os.path.join(test_out_annotation_dir, 'annotations.txt')
        with open(out_annotation_path, 'w') as f:
            for image_ppt in self.ppts_test:            
                f.writelines([image_ppt[0][0], ' ', ' '.join(str(','.join(str(ppt) for ppt in ppts[1:])) 
                                                                for ppts in image_ppt), '\n'])
        #write to txt file
        validation_out_annotation_dir = self.valid_path('validation')
        out_annotation_path = os.path.join(validation_out_annotation_dir, 'annotations.txt')
        with open(out_annotation_path, 'w') as f:
            for image_ppt in self.ppts_valid:            
                f.writelines([image_ppt[0][0], ' ', ' '.join(str(','.join(str(ppt) for ppt in ppts[1:])) 
                                                                for ppts in image_ppt), '\n'])
        print('All done!') 

    def rectjson_to_xml(self):
        print('Starting conversion...')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        tree = ET.parse(self.xml_template)
        root = tree.getroot()
        for image_ppt in self.ppts_train:   
            img_path, img_width, img_height, img_depth = image_ppt[0][:4]         
            #write to xml file
            train_out_annotation_dir = self.valid_path('train')
            out_annotation_path = os.path.splitext(os.path.join(train_out_annotation_dir, img_path[8:]))[0] + '.xml'
            #modify xml template
            folder = root.find('folder')
            folder.text = train_out_annotation_dir

            fname = root.find('filename')
            fname.text = img_path

            src = root.find('source')
            database = src.find('database')
            database.text = self.database

            size = root.find('size')
            width = size.find('width')
            width.text = str(img_width)
            height = size.find('height')
            height.text = str(img_height)
            depth = size.find('depth')
            depth.text = str(img_depth)

            for ppt in image_ppt:
                #append new object
                obj = ET.SubElement(root, 'object')

                name = ET.SubElement(obj, 'name')
                name.text = str(ppt[4])

                pose = ET.SubElement(obj, 'pose')
                pose.text = 'Unspecified'

                bndbox = ET.SubElement(obj, 'bndbox')

                xmin = ET.SubElement(bndbox, 'xmin')
                xmin.text = str(int(ppt[6]))
                xmax = ET.SubElement(bndbox, 'xmax')
                xmax.text = str(int(ppt[7]))
                ymin = ET.SubElement(bndbox, 'ymin')
                ymin.text = str(int(ppt[8]))
                ymax = ET.SubElement(bndbox, 'ymax')
                ymax.text = str(int(ppt[9]))

            #save annotation to xml file
            tree.write(out_annotation_path)
        
        for image_ppt in self.ppts_test:   
            img_path, img_width, img_height, img_depth = image_ppt[0][:4]         
            #write to xml file
            test_out_annotation_dir = self.valid_path('test')
            out_annotation_path = os.path.splitext(os.path.join(test_out_annotation_dir, img_path[8:]))[0] + '.xml'
            #modify xml template
            folder = root.find('folder')
            folder.text = test_out_annotation_dir

            fname = root.find('filename')
            fname.text = img_path

            src = root.find('source')
            database = src.find('database')
            database.text = self.database

            size = root.find('size')
            width = size.find('width')
            width.text = str(img_width)
            height = size.find('height')
            height.text = str(img_height)
            depth = size.find('depth')
            depth.text = str(img_depth)

            for ppt in image_ppt:
                #append new object
                obj = ET.SubElement(root, 'object')

                name = ET.SubElement(obj, 'name')
                name.text = str(ppt[4])

                pose = ET.SubElement(obj, 'pose')
                pose.text = 'Unspecified'

                bndbox = ET.SubElement(obj, 'bndbox')

                xmin = ET.SubElement(bndbox, 'xmin')
                xmin.text = str(int(ppt[6]))
                xmax = ET.SubElement(bndbox, 'xmax')
                xmax.text = str(int(ppt[7]))
                ymin = ET.SubElement(bndbox, 'ymin')
                ymin.text = str(int(ppt[8]))
                ymax = ET.SubElement(bndbox, 'ymax')
                ymax.text = str(int(ppt[9]))

            #save annotation to xml file
            tree.write(out_annotation_path)
        
        for image_ppt in self.ppts_valid:   
            img_path, img_width, img_height, img_depth = image_ppt[0][:4]         
            #write to xml file
            validation_out_annotation_dir = self.valid_path('validation')
            out_annotation_path = os.path.splitext(os.path.join(validation_out_annotation_dir, img_path[8:]))[0] + '.xml'
            #modify xml template
            folder = root.find('folder')
            folder.text = validation_out_annotation_dir

            fname = root.find('filename')
            fname.text = img_path

            src = root.find('source')
            database = src.find('database')
            database.text = self.database

            size = root.find('size')
            width = size.find('width')
            width.text = str(img_width)
            height = size.find('height')
            height.text = str(img_height)
            depth = size.find('depth')
            depth.text = str(img_depth)

            for ppt in image_ppt:
                #append new object
                obj = ET.SubElement(root, 'object')

                name = ET.SubElement(obj, 'name')
                name.text = str(ppt[4])

                pose = ET.SubElement(obj, 'pose')
                pose.text = 'Unspecified'

                bndbox = ET.SubElement(obj, 'bndbox')

                xmin = ET.SubElement(bndbox, 'xmin')
                xmin.text = str(int(ppt[6]))
                xmax = ET.SubElement(bndbox, 'xmax')
                xmax.text = str(int(ppt[7]))
                ymin = ET.SubElement(bndbox, 'ymin')
                ymin.text = str(int(ppt[8]))
                ymax = ET.SubElement(bndbox, 'ymax')
                ymax.text = str(int(ppt[9]))

            #save annotation to xml file
            tree.write(out_annotation_path)
        print('All done!')

    def rectjson_to_kerasRetinanetCsv(self):
        print('Starting conversion...')
        #write to csv file
        train_out_annotation_dir = self.valid_path('train')
        test_out_annotation_dir = self.valid_path('test')
        validation_out_annotation_dir = self.valid_path('validation')
        out_annotation_path_train = os.path.join(train_out_annotation_dir, 'annotations.csv')
        out_annotation_path_test = os.path.join(test_out_annotation_dir, 'annotations.csv')
        out_annotation_path_valid = os.path.join(validation_out_annotation_dir, 'annotations.csv')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        with open(out_annotation_path_train, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_train:            
                writer.writerows(image_ppt)
        with open(out_annotation_path_test, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_test:            
                writer.writerows(image_ppt)
        with open(out_annotation_path_valid, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_valid:            
                writer.writerows(image_ppt)

        print('All done!')

    def rectjson_to_tfCsv(self):
        print('Starting conversion...')
        #write to csv file
        train_out_annotation_dir = self.valid_path('train')
        test_out_annotation_dir = self.valid_path('test')
        validation_out_annotation_dir = self.valid_path('validation')
        out_annotation_path_train = os.path.join(train_out_annotation_dir, 'annotations.csv')
        out_annotation_path_test = os.path.join(test_out_annotation_dir, 'annotations.csv')
        out_annotation_path_valid = os.path.join(validation_out_annotation_dir, 'annotations.csv')
        #get all image info
        self.extract_info_from_json()
        #split to train, test, validation
        self.ppts_train, self.ppts_test, self.ppts_valid = self.split(self.ppts, self.train_ratio, self.test_ratio, self.valid_ratio)
        with open(out_annotation_path_train, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_train:  
                writer.writerows(image_ppt)
        with open(out_annotation_path_test, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_test:            
                writer.writerows(image_ppt)
        with open(out_annotation_path_valid, 'w') as f:
            writer = csv.writer(f)

            for image_ppt in self.ppts_valid:            
                writer.writerows(image_ppt)
        print('All done!')
        



    

    


#this module uploads user annotation folder (test, validation and train) as a zip file
# from .s3API import s3API
from zipfile import ZipFile
import requests
from json import loads
# class Upload():
#     def __init__(self, user_id, project_id, input_filepath, filename):
#         self.user_id = user_id
#         self.project_id = project_id
#         self.input_filepath = input_filepath
#         self.filename = filename

#         #upload the data
#         self.upload()

#     def upload(self):
#         """This method uploads data to the aws storage
#         of this specified project_id"""
#         url = s3API.uploader(self.input_filepath, self.filename)['url'] + str(self.filename)
#         print("presigned url of uploaded file: ", url)

class Upload():
    def __init__(self, user_id, project_id, user_token, train_path, test_path, validation_path, label_to_id_path):
        self.user_id = user_id
        self.project_id = project_id
        self.user_token = user_token
        self.ann_file_paths = [train_path, test_path, validation_path, label_to_id_path]

    def compress_annotations(self, zip_path):
        #write files in ann_file_paths to a zipfile
        with ZipFile(zip_path, 'w') as zip:
            for file in self.ann_file_paths:
                zip.write(file)
        return zip_path

    def upload(self):
        #this method zips, uploads zipped annotation file to s3 and updates db url
        #zip projects directory
        zip_path = self.compress_annotations(zip_path='annotations.zip')
        #post annotation to endpoint   
        url = 'http://164.92.64.23/api/v1/upload/project/export'
        data={'userId':self.user_id, 'projectId':self.project_id, 'fileType':'files'}
        file = {'file': ('annotation', open(zip_path, 'rb'))}        
        headers={'Authorization':self.user_token}
        response = requests.post(url, files=file, data=data, headers=headers)
        if response.ok:
            print("Upload completed successfully!")
            response_details = loads(response.text)['data']
            #extract url from response_details
            url = response_details['imagePath']
            print('Uploaded successfully to {}'.format(url))
            
        else:
            print("Something went wrong!")
            print(response.text)



        